#! /usr/bin/env python
from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

#chars=" qwertzuiopü+asdfghjklöä#yxcvbnm,.-QWERTZUIOPÜ*ASDFGHJKLÖÄ'YXCVBNM;:_1234567890ß!\"§$%&/()=?"
chars=" qwertzuiop+asdfghjklöä#yxcvbnm,.-QWERTZUIOP*ASDFGHJKLÖÄ'YXCVBNM;:_1234567890ß!\"§$%&/()=?"
#tileratio=5.0/10 #for the ratio   x y
tileratio=(38.0/18.0) / (30.0/7)

#asciiresolution_max=[78,65] #maximum, in characters xy
asciiresolution_max=[60,30] #maximum, in characters xy


def main():
    import sys
    if len(sys.argv) != 2:
        print("Usage: asciiart filename")
        sys.exit(1)
    imagefilename=sys.argv[1]

    asciiresolution=asciiresolution_max

    im = Image.open(imagefilename)
    rgb_im = im.convert('RGB')

    imwidth=rgb_im.size[0]
    imheight=rgb_im.size[1]

    tilewidth=imwidth/asciiresolution_max[0]
    tileheight=tilewidth/tileratio
    asciiresolution[1]=int(imheight/tileheight) #new resolution



    if tileheight*asciiresolution_max[1]>imheight: #does not fit
        print("using height as reference")
        tileheight=imheight/asciiresolution_max[1]
        tilewidth=tileheight*tileratio
        asciiresolution[0]=int(imwidth/tilewidth) #new resolution

    print("using tilesize="+str((tilewidth,tileheight)))

    asciiimages=[]
    for c in chars:
        asciiimg = Image.new('RGB', (int(tilewidth),int(tileheight)), (255,255,255))
        draw = ImageDraw.Draw(asciiimg)
        # font = ImageFont.truetype(<font-file>, <font-size>)
        font = ImageFont.truetype("Arial.ttf", int(tileheight))
        # draw.text((x, y),"Sample Text",(r,g,b))
        draw.text((0, 0),c,(0,0,0),font=font)
        #asciiimg.save('sample'+currentchar+'.png')
        asciiimages.append(asciiimg)

    output=""

    for ay in range(asciiresolution[1]):
        print(str(ay)+" / "+str(asciiresolution[1]))
        outputline="" #row
        for ax in range(asciiresolution[0]):
            upperleftx=int(ax*tilewidth)
            upperlefty=int(ay*tileheight)
            lowerrightx=int((ax+1)*tilewidth-1)
            lowerrighty=int((ay+1)*tileheight-1)

            bestcharindex=0
            bestchardiff=-1

            for currentcharindex,currentchar in enumerate(chars):
                asciiimg=asciiimages[currentcharindex]

                diff=0
                for ytile in range(upperlefty,lowerrighty):
                    for xtile in range(upperleftx,lowerrightx):
                        #print((xtile,ytile))
                        r, g, b = rgb_im.getpixel((xtile,ytile))
                        asciir,asciig,asciib=asciiimg.getpixel((xtile-upperleftx,ytile-upperlefty))

                        brightness=(r+g+b)/3
                        asciibrightness=(asciir+asciig+asciib)/3

                        diff+=abs(brightness-asciibrightness)


                if diff<bestchardiff or bestchardiff<0:
                    bestchardiff=diff
                    bestcharindex=currentcharindex

            #print("bestchar "+str(bestcharindex)+" "+chars[bestcharindex])
            outputline+=chars[bestcharindex]
        output+=outputline.rstrip()
        output+="\n"

    print(output)





if __name__ == "__main__":
    main()
